#!/usr/bin/perl

use warnings;
use strict;

use SDL::App;
use SDL::Rect;
use SDL::Color;
use SDL::Constants;
use SDL::Event;
use Math::CatmullRom;
#use Algorithm::Line::Bresenham;

use Carp qw(cluck);
use Data::Dump qw(dump);

our $debug = 0;

my ( $w, $h ) = ( 800, 480 );
my $mouse_trashold = 10;
my $max_path_length = 200;

sub debug {
	return unless $debug;
	my ($package, $filename, $line) = caller;
	warn '# ', dump( @_ ), " $filename +$line\n";
}

our $app = SDL::App->new(
	-width  => $w,
	-height => $h,
	-depth  => 16,
#	-flags=>SDL_DOUBLEBUF | SDL_HWSURFACE | SDL_HWACCEL,
	-title  => 'Trace mouse',
);

our $event = SDL::Event->new;

our $mouse_down = 0;

our $white = SDL::Color->new( 0xff, 0xff, 0xff );
our $red   = SDL::Color->new( 0xff, 0x00, 0x00 );
our $black = SDL::Color->new( 0x00, 0x00, 0x00 );

my ( $last_x, $last_y ) = ( 0,0 );

our @path;

sub curve {
	return unless $#path > 4;
	my $curve = Math::CatmullRom->new( splice @path, 0, $#path + $#path / 2 );
	my @curve = $curve->curve( $mouse_trashold * 10 );
	debug 'curve' => @curve;

	my $i = 0;
	while ( $i < $#curve ) {
		$app->pixel( int($curve[$i++]), int($curve[$i++]), $red );
	}
	$app->sync;
}

sub handle_events {

	while ( $event->wait ) {
		my $type = $event->type();

		if ( $type == SDL_MOUSEBUTTONDOWN() ) {
			debug 'mouse down', $event->button_x, $event->button_y;
			$mouse_down = 1;
		} elsif ( $type == SDL_MOUSEBUTTONUP() ) {
			debug 'mouse up', $event->button_x, $event->button_y;
			$mouse_down = 0;
			curve;
		} elsif ( $type == SDL_QUIT() ) {
			exit;
		} elsif ( $type == SDL_KEYDOWN() ) {
			my $key = $event->key_name;
			debug 'key down', $key;
			exit if $key =~ m/^[xq]$/;
			if ( $key eq 's' ) { # XXX draw curve
				curve;
			} elsif ( $key eq 'backspace' ) { # XXX clean screen
				@path = ();
				my $rect = SDL::Rect->new( -x => 0, -y => 0, -w => $w, -h => $h );
				$app->fill( $rect, $black );
				$app->update( $rect );
			} elsif ( $key eq 'd' ) { # XXX toggle debug
				$debug = ! $debug;
				warn "debug $debug\n";
			} else {
				warn "unknown key $key";
			}
		} elsif ( $type == SDL_KEYUP() ) {
			debug 'key up', $event->key_name;
		} elsif ( $type == SDL_MOUSEMOTION() ) {
			my ( $x, $y ) = ( $event->motion_x, $event->motion_y );
			debug 'mouse', $mouse_down, $x, $y;
			my $dx = abs( $last_x - $x );
			my $dy = abs( $last_y - $y );
			if ( $mouse_down && ( $dx > $mouse_trashold || $dy > $mouse_trashold ) ) {
				if ( $#path < $max_path_length ) {
					push @path, $x, $y;
					my $rect = SDL::Rect->new( -x => $event->motion_x - 1, -y => $event->motion_y -1 , -w => 3, -h => 3 );
					$app->fill( $rect, $white );
					$app->update( $rect );
					$last_x = $x;
					$last_y = $y;
				} else {
					$mouse_down = 0;
					curve;
				}
			}
		} else {
			warn "unknown type $type\n";
		}
	}
};

while(1) {
	handle_events;
	$app->sync;
	$app->delay(1);
}

1;
